@@ -1,35 +1,50 @@
-# constructor of the matrix representation
+# constructor of the matrix lavoptions$representation
 #
 # initial version: YR 22/11/2010
 # - YR 14 Jan 2014: moved to lav_model.R
 # - YR 18 Nov 2014: more efficient handling of linear equality constraints
 # - YR 02 Dec 2014: allow for bare-minimum parameter tables
+# - YR 25 Jan 2017: collect options in lavoptions
 
-# construct MATRIX representation of the model
+# construct MATRIX lavoptions$representation of the model
 lav_model <- function(lavpartable      = NULL,
-                      representation   = "LISREL",
-                      conditional.x    = FALSE,
+                      lavoptions       = NULL,
                       th.idx           = list(),
-                      parameterization = "delta",
-                      link             = "logit",
-                      control          = list(),
-                      debug            = FALSE) {
+                      cov.x            = list(),
+                      mean.x           = list()) { # for conditional.x only
+                                                   # (not really needed,
+                                                   #  but as a failsafe)
 
     # handle bare-minimum partables
     lavpartable <- lav_partable_complete(lavpartable)
- 
+
     # global info from user model
-    ngroups <- max(lavpartable$group)
+    nblocks <- lav_partable_nblocks(lavpartable)
+    ngroups <- lav_partable_ngroups(lavpartable)
     meanstructure <- any(lavpartable$op == "~1")
     categorical <- any(lavpartable$op == "|")
-    if(categorical) meanstructure <- TRUE
-    group.w.free <- any(lavpartable$lhs == "group" & lavpartable$op == "%")
+    if(categorical) {
+        meanstructure <- TRUE
 
+        # handle th.idx if length(th.idx) != nblocks
+        if(nblocks != length(th.idx)) {
+            th.idx <- rep(th.idx, each = nblocks)
+        }
+
+    }
+    group.w.free <- any(lavpartable$lhs == "group" & lavpartable$op == "%")
+    multilevel <- FALSE
+    if(!is.null(lavpartable$level)) {
+        nlevels <- lav_partable_nlevels(lavpartable)
+        if(nlevels > 1L) {
+            multilevel <- TRUE
+        }
+    }
 
     # handle variable definitions and (in)equality constraints
     CON <- lav_constraints_parse(partable = lavpartable,
                                  constraints = NULL,
-                                 debug = debug)
+                                 debug = lavoptions$debug)
 
     # handle *linear* equality constraints special
     if(CON$ceq.linear.only.flag) {
@@ -43,20 +58,20 @@ lav_model <- function(lavpartable      = NULL,
     }
 
     # select model matrices
-    if(representation == "LISREL") {
-        REP <- representation.LISREL(lavpartable, target=NULL, extra=TRUE)
+    if(lavoptions$representation == "LISREL") {
+        REP <- representation.LISREL(lavpartable, target = NULL, extra = TRUE)
     } else {
         stop("lavaan ERROR: only representation \"LISREL\" has been implemented.")
     }
-    if(debug) print(REP)
+    if(lavoptions$debug) print(REP)
 
     # FIXME: check for non-existing parameters
-    bad.idx <- which(REP$mat == "" & 
+    bad.idx <- which(REP$mat == "" &
                      !lavpartable$op %in% c("==","<",">",":="))
 
     if(length(bad.idx) > 0L) {
-        
-        label <- paste(lavpartable$lhs[bad.idx[1]], 
+
+        label <- paste(lavpartable$lhs[bad.idx[1]],
                        lavpartable$op[bad.idx[1]],
                        lavpartable$rhs[bad.idx[1]], sep = " ")
         stop("lavaan ERROR: parameter is not defined: ", label)
@@ -73,33 +88,34 @@ lav_model <- function(lavpartable      = NULL,
     m.free.idx <- m.user.idx <- vector(mode="list", length=nG)
     x.free.idx <- x.user.idx <- vector(mode="list", length=nG)
 
-    # prepare ngroups-sized slots
-    nvar <- integer(ngroups)
+    # prepare nblocks-sized slots
+    nvar <- integer(nblocks)
     nmat <- unlist(attr(REP, "mmNumber"))
-    num.idx <- vector("list", length=ngroups)
-    nexo <- integer(ngroups)
-    ov.x.dummy.ov.idx <- vector(mode="list", length=ngroups)
-    ov.x.dummy.lv.idx <- vector(mode="list", length=ngroups)
-    ov.y.dummy.ov.idx <- vector(mode="list", length=ngroups)
-    ov.y.dummy.lv.idx <- vector(mode="list", length=ngroups)
+    num.idx <- vector("list", length=nblocks)
+    nexo <- integer(nblocks)
+    ov.x.dummy.ov.idx <- vector(mode="list", length=nblocks)
+    ov.x.dummy.lv.idx <- vector(mode="list", length=nblocks)
+    ov.y.dummy.ov.idx <- vector(mode="list", length=nblocks)
+    ov.y.dummy.lv.idx <- vector(mode="list", length=nblocks)
 
     offset <- 0L
-    for(g in 1:ngroups) {
+    for(g in 1:nblocks) {
 
-        # observed and latent variables for this group
-        ov.names <- vnames(lavpartable, "ov", group=g)
-        ov.names.nox <- vnames(lavpartable, "ov.nox", group=g)
-        ov.names.x <- vnames(lavpartable, "ov.x", group=g)
+        # observed and latent variables for this block
+        ov.names <-     lav_partable_vnames(lavpartable, "ov",     block = g)
+        ov.names.nox <- lav_partable_vnames(lavpartable, "ov.nox", block = g)
+        ov.names.x <-   lav_partable_vnames(lavpartable, "ov.x",   block = g)
         nexo[g] <- length(ov.names.x)
-        ov.num <- vnames(lavpartable, "ov.num", group=g)
-        if(conditional.x) {
+        ov.num <-       lav_partable_vnames(lavpartable, "ov.num", block = g)
+        if(lavoptions$conditional.x) {
             nvar[g] <- length(ov.names.nox)
+            num.idx[[g]] <- which(ov.names.nox %in% ov.num)
         } else {
             nvar[g] <- length(ov.names)
+            num.idx[[g]] <- which(ov.names %in% ov.num)
         }
-        num.idx[[g]] <- match(ov.num, ov.names.nox)
 
-        # model matrices for this group
+        # model matrices for this block
         mmNumber    <- attr(REP, "mmNumber")[[g]]
         mmNames     <- attr(REP, "mmNames")[[g]]
         mmSymmetric <- attr(REP, "mmSymmetric")[[g]]
@@ -124,7 +140,7 @@ lav_model <- function(lavpartable      = NULL,
             dimNames[[offset]] <- mmDimNames[[mm]]
 
             # select elements for this matrix
-            idx <- which(lavpartable$group == g & REP$mat == mmNames[mm]) 
+            idx <- which(lavpartable$block == g & REP$mat == mmNames[mm])
 
             # create empty `pattern' matrix
             # FIXME: one day, we may want to use sparse matrices...
@@ -145,7 +161,7 @@ lav_model <- function(lavpartable      = NULL,
             # 2. if equality constraints, unconstrained free parameters
             #    -> to be used in lav_model_gradient
             #if(CON$ceq.linear.only.flag) {
-            #    tmp[ cbind(REP$row[idx], 
+            #    tmp[ cbind(REP$row[idx],
             #               REP$col[idx]) ] <- lavpartable$unco[idx]
             #    if(mmSymmetric[mm]) {
             #        # NOTE: we assume everything is in the UPPER tri!
@@ -165,7 +181,7 @@ lav_model <- function(lavpartable      = NULL,
             }
             m.user.idx[[offset]] <-     which(tmp > 0)
             x.user.idx[[offset]] <- tmp[which(tmp > 0)]
-          
+
             # 4. now assign starting/fixed values
             # create empty matrix
             # FIXME: again, we may want to use sparse matrices here...
@@ -176,8 +192,21 @@ lav_model <- function(lavpartable      = NULL,
                 T <- t(tmp); tmp[lower.tri(tmp)] <- T[lower.tri(T)]
             }
 
+            # 4b. override with cov.x (if conditional.x = TRUE)
+            # new in 0.6-1
+            # shouldn't be needed, if lavpartable$start contains cov.x values
+            if(mmNames[mm] == "cov.x") {
+                tmp <- cov.x[[g]]
+            }
+            # 4c. override with mean.x (if conditional.x = TRUE)
+            # new in 0.6-1
+            # shouldn't be needed, if lavpartable$start contains mean.x values
+            if(mmNames[mm] == "mean.x") {
+                tmp <- as.matrix(mean.x[[g]])
+            }
+
             # representation specific stuff
-            if(representation == "LISREL" && mmNames[mm] == "lambda") { 
+            if(lavoptions$representation == "LISREL" && mmNames[mm] == "lambda") {
                 ov.dummy.names.nox <- attr(REP, "ov.dummy.names.nox")[[g]]
                 ov.dummy.names.x   <- attr(REP, "ov.dummy.names.x")[[g]]
                 ov.dummy.names <- c(ov.dummy.names.nox, ov.dummy.names.x)
@@ -198,36 +227,59 @@ lav_model <- function(lavpartable      = NULL,
             }
 
             # representation specific
-            if(representation == "LISREL" && mmNames[mm] == "delta") {
+            if(lavoptions$representation == "LISREL" && mmNames[mm] == "delta") {
                 # only categorical values are listed in the lavpartable
                 # but all remaining values should be 1.0
                 idx <- which(tmp[,1L] == 0.0)
-                tmp[idx,1L] <- 1.0    
+                tmp[idx,1L] <- 1.0
             }
-            
+
             # assign matrix to GLIST
             GLIST[[offset]] <- tmp
         } # mm
     } # g
 
     # fixed.x parameters?
-    fixed.x <- any(lavpartable$exo > 0L & lavpartable$free == 0L) 
-    if(categorical) {
-        fixed.x <- TRUE
+    #fixed.x <- any(lavpartable$exo > 0L & lavpartable$free == 0L)
+    #if(categorical) {
+    #    fixed.x <- TRUE
+    #}
+
+    # dirty hack to mimic MUML
+    if(!is.null(lavoptions$tech.muml.scale)) {
+        warning("lavaan WARNING: using muml scale in group 2")
+
+        # find matrix
+        lambda.idx <- which(names(GLIST) == "lambda")[2L]
+
+        # find rows/cols
+        B.names <- paste0("b", ov.names) ## ad-hoc assumption!!!
+        COLS <-  match(B.names, LV.names)
+        ROWS <- seq_len(nvar[2])
+        stopifnot(length(COLS) == length(ROWS))
+        GLIST[[ lambda.idx ]][ cbind(ROWS, COLS) ] <- lavoptions$tech.muml.scale
     }
 
+    # which free parameters are observed variances?
+    ov.names <- vnames(lavpartable, "ov")
+    x.free.var.idx <- lavpartable$free[ lavpartable$free &
+                                        #!duplicated(lavpartable$free) &
+                                        lavpartable$lhs %in% ov.names &
+                                        lavpartable$op == "~~" &
+                                        lavpartable$lhs == lavpartable$rhs ]
 
-    Model <- new("Model",
+    Model <- new("lavModel",
                  GLIST=GLIST,
                  dimNames=dimNames,
                  isSymmetric=isSymmetric,
                  mmSize=mmSize,
-                 representation=representation,
+                 representation=lavoptions$representation,
                  meanstructure=meanstructure,
                  categorical=categorical,
-                 link=link,
-                 control=control,
-                 ngroups=ngroups,
+                 multilevel=multilevel,
+                 link=lavoptions$link,
+                 nblocks=nblocks,
+                 ngroups=ngroups, # breaks rsem????
                  group.w.free=group.w.free,
                  nmat=nmat,
                  nvar=nvar,
@@ -238,6 +290,7 @@ lav_model <- function(lavpartable      = NULL,
                  nx.user=max(lavpartable$id),
                  m.free.idx=m.free.idx,
                  x.free.idx=x.free.idx,
+                 x.free.var.idx=x.free.var.idx,
                  #m.unco.idx=m.unco.idx,
                  #x.unco.idx=x.unco.idx,
                  m.user.idx=m.user.idx,
@@ -269,18 +322,20 @@ lav_model <- function(lavpartable      = NULL,
                  con.lambda          = con.lambda,
 
                  nexo                = nexo,
-                 fixed.x             = fixed.x,
-                 conditional.x       = conditional.x,
+                 fixed.x             = lavoptions$fixed.x,
+                 conditional.x       = lavoptions$conditional.x,
                  #x.idx               = x.idx,
-                 parameterization    = parameterization,
+                 parameterization    = lavoptions$parameterization,
 
                  ov.x.dummy.ov.idx   = ov.x.dummy.ov.idx,
                  ov.x.dummy.lv.idx   = ov.x.dummy.lv.idx,
                  ov.y.dummy.ov.idx   = ov.y.dummy.ov.idx,
-                 ov.y.dummy.lv.idx   = ov.y.dummy.lv.idx)
+                 ov.y.dummy.lv.idx   = ov.y.dummy.lv.idx,
 
-    if(debug) {
-         cat("lavaan DEBUG: lavaanModel\n")
+                 estimator           = lavoptions$estimator)
+
+    if(lavoptions$debug) {
+         cat("lavaan lavoptions$debug: lavaanModel\n")
          print( str(Model) )
          print( Model@GLIST )
     }
