@@ -8,7 +8,7 @@
 # starting values, using the sample data
 
 lav_start <- function(start.method    = "default",
-                      lavpartable     = NULL, 
+                      lavpartable     = NULL,
                       lavsamplestats  = NULL,
                       model.type      = "sem",
                       mimic           = "lavaan",
@@ -19,12 +19,16 @@ lav_start <- function(start.method    = "default",
 
     # categorical?
     categorical <- any(lavpartable$op == "|")
-    
+
     # conditional.x?
     conditional.x <- any(lavpartable$exo == 1L & lavpartable$op == "~")
     #ord.names <- unique(lavpartable$lhs[ lavpartable$op == "|" ])
 
+    # nlevels?
+    nlevels <- lav_partable_nlevels(lavpartable)
+
     # shortcut for 'simple'
+    # be we should also take care of the 'fixed.x' issue
     if(identical(start.method, "simple")) {
         start <- numeric( length(lavpartable$ustart) )
         start[ which(lavpartable$op == "=~") ] <- 1.0
@@ -52,7 +56,7 @@ lav_start <- function(start.method    = "default",
         start.method. <- tolower(start.method)
         if(start.method. == "default") {
             # nothing to do
-        } else if(start.method. %in% c("simple", "lavaan", "mplus")) { 
+        } else if(start.method. %in% c("simple", "lavaan", "mplus")) {
             start.initial <- start.method.
         } else {
             stop("lavaan ERROR: unknown value for start argument")
@@ -80,14 +84,14 @@ lav_start <- function(start.method    = "default",
         } else {
             stop("lavaan ERROR: problem with start argument: could not find est/start column in model list")
         }
-    }   
+    }
 
 
     # global settings
     # 0. everyting is zero
     start <- numeric( length(lavpartable$ustart) )
 
-    # 1. =~ factor loadings: 
+    # 1. =~ factor loadings:
     if(categorical) {
         # if std.lv=TRUE, more likely initial Sigma.hat is positive definite
         # 0.8 is too large
@@ -96,116 +100,61 @@ lav_start <- function(start.method    = "default",
         start[ which(lavpartable$op == "=~") ] <- 1.0
     }
 
-    # 2. residual lv variances for latent variables
-    lv.names    <- vnames(lavpartable, "lv") # all blocks
+    # 2. (residual) lv variances for latent variables
+    lv.names    <- vnames(lavpartable, "lv") # all groups
     lv.var.idx <- which(lavpartable$op == "~~"        &
                         lavpartable$lhs %in% lv.names &
                         lavpartable$lhs == lavpartable$rhs)
     start[lv.var.idx] <- 0.05
-
+    #start[lv.var.idx] <- 0.5 # new in 0.6-2? (for optim.parscale = "stand")
 
     # 3. latent response scales (if any)
     delta.idx <- which(lavpartable$op == "~*~")
     start[delta.idx] <- 1.0
 
 
-    # block-specific settings
-    nblocks <- lav_partable_nblocks(lavpartable)
+    # group-specific settings
+    ngroups <- lav_partable_ngroups(lavpartable)
 
-    for(g in 1:nblocks) {
+    # for now, if no group column, add one (again), until we rewrite
+    # this function to handle block/group hybrid settings
+    if(is.null(lavpartable$group) && ngroups == 1L) {
+        lavpartable$group <- rep(1L, length(lavpartable$lhs))
+        lavpartable$group[ lavpartable$block == 0L] <- 0L
+    }
 
-        # info from user model for this block
+
+    for(g in 1:ngroups) {
+
+        # group values (not necessarily 1,2,... anymore)
+        group.values <- lav_partable_group_values(lavpartable)
+
+        # info from user model for this group
         if(conditional.x) {
-            ov.names     <- vnames(lavpartable, "ov.nox", block = g)
+            ov.names     <- vnames(lavpartable, "ov.nox", group = group.values[g])
         } else {
-            ov.names     <- vnames(lavpartable, "ov", block = g)
+            ov.names     <- vnames(lavpartable, "ov", group = group.values[g])
         }
         if(categorical) {
-            ov.names.num <- vnames(lavpartable, "ov.num", block = g)
-            ov.names.ord <- vnames(lavpartable, "ov.ord", block = g)
+            ov.names.num <- vnames(lavpartable, "ov.num", group = group.values[g])
+            ov.names.ord <- vnames(lavpartable, "ov.ord", group = group.values[g])
         } else {
             ov.names.num <- ov.names
         }
-        lv.names    <- vnames(lavpartable, "lv",   block = g)
-        ov.names.x  <- vnames(lavpartable, "ov.x", block = g)
+        lv.names    <- vnames(lavpartable, "lv",   group = group.values[g])
+        ov.names.x  <- vnames(lavpartable, "ov.x", group = group.values[g])
 
-        # g1) factor loadings
-        if(start.initial %in% c("lavaan", "mplus") && 
-           model.type %in% c("sem", "cfa") &&
-           #!categorical &&
-           sum( lavpartable$ustart[ lavpartable$op == "=~" & lavpartable$block == g],
-                                   na.rm=TRUE) == length(lv.names) ) {
-            # only if all latent variables have a reference item,
-            # we use the fabin3 estimator (2sls) of Hagglund (1982)
-            # per factor
-            # 9 Okt 2013: if only 2 indicators, we use the regression
-            # coefficient (y=marker, x=2nd indicator)
-            for(f in lv.names) {
-                free.idx <- which( lavpartable$lhs == f & lavpartable$op == "=~"
-                                                 & lavpartable$block == g
-                                                 & lavpartable$free > 0L)
-                 
-                user.idx <- which( lavpartable$lhs == f & lavpartable$op == "=~" 
-                                                 & lavpartable$block == g )
-                # no second order
-                if(any(lavpartable$rhs[user.idx] %in% lv.names)) next
+        # just for the nlevels >1 case
+        ov.names <- unique(unlist(ov.names))
+        ov.names.num <- unique(unlist(ov.names.num))
+        lv.names <- unique(unlist(lv.names))
+        ov.names.x <- unique(unlist(ov.names.x))
 
-                # get observed indicators for this latent variable
-                ov.idx <- match(lavpartable$rhs[user.idx], ov.names)
-                if(length(ov.idx) > 2L && !any(is.na(ov.idx))) {
-                    if(lavsamplestats@missing.flag) {
-                        COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
-                                                                    ov.idx]
-                    } else {
-                        if(conditional.x) {
-                            COV <- lavsamplestats@res.cov[[g]][ov.idx,ov.idx]
-                        } else {
-                            COV <- lavsamplestats@cov[[g]][ov.idx,ov.idx]
-                        }
-                    }
-                    start[user.idx] <- fabin3.uni(COV)
-                } else if(length(free.idx) == 1L && length(ov.idx) == 2L) {
-                    if(conditional.x) {
-                        REG2 <- ( lavsamplestats@res.cov[[g]][ov.idx[1],
-                                                              ov.idx[2]] /
-                                  lavsamplestats@res.cov[[g]][ov.idx[1],
-                                                              ov.idx[1]] )
-                    } else {
-                        REG2 <- ( lavsamplestats@cov[[g]][ov.idx[1],
-                                                          ov.idx[2]] /
-                                  lavsamplestats@cov[[g]][ov.idx[1],
-                                                          ov.idx[1]] )
-                    }
-                    start[free.idx] <- REG2
-                }
 
-                # standardized?
-                var.f.idx <- which(lavpartable$lhs == f & lavpartable$op == "~~" &
-                                   lavpartable$rhs == f)
-                if(length(var.f.idx) > 0L && 
-                   lavpartable$free[var.f.idx] == 0 &&
-                   lavpartable$ustart[var.f.idx] == 1) {
-                   # make sure factor loadings are between -0.7 and 0.7
-                    x <- start[user.idx]
-                    start[user.idx] <- (x / max(abs(x))) * 0.7
-                }
-            }
-        }
-
-        if(model.type == "unrestricted") {
-           # fill in 'covariances' from lavsamplestats
-            cov.idx <- which(lavpartable$block == g             &
-                             lavpartable$op    == "~~"          &
-                             lavpartable$lhs != lavpartable$rhs)
-            lhs.idx <- match(lavpartable$lhs[cov.idx], ov.names)
-            rhs.idx <- match(lavpartable$rhs[cov.idx], ov.names)
-            start[cov.idx] <- lavsamplestats@cov[[g]][ cbind(lhs.idx, rhs.idx) ]
-        }
-
-        # 2g) residual ov variances (including exo, to be overriden)
-        ov.var.idx <- which(lavpartable$block == g             & 
-                            lavpartable$op    == "~~"          & 
-                            lavpartable$lhs %in% ov.names.num  & 
+        # residual ov variances (including exo/ind, to be overriden)
+        ov.var.idx <- which(lavpartable$group == group.values[g]             &
+                            lavpartable$op    == "~~"          &
+                            lavpartable$lhs %in% ov.names.num  &
                             lavpartable$lhs == lavpartable$rhs)
         sample.var.idx <- match(lavpartable$lhs[ov.var.idx], ov.names)
         if(model.type == "unrestricted") {
@@ -216,7 +165,7 @@ lav_start <- function(start.method    = "default",
                     start[ov.var.idx] <-
                        (1.0 - 0.50)*lavsamplestats@res.var[[1L]][sample.var.idx]
                 } else {
-                    start[ov.var.idx] <- 
+                    start[ov.var.idx] <-
                        (1.0 - 0.50)*lavsamplestats@var[[1L]][sample.var.idx]
                 }
             } else {
@@ -224,15 +173,110 @@ lav_start <- function(start.method    = "default",
                     start[ov.var.idx] <-
                   (1.0 - 0.50)*diag(lavsamplestats@res.cov[[g]])[sample.var.idx]
                 } else {
-                    start[ov.var.idx] <- 
+                    start[ov.var.idx] <-
                     (1.0 - 0.50)*diag(lavsamplestats@cov[[g]])[sample.var.idx]
                 }
             }
         }
 
-        # variances of ordinal variables - set to 1.0     
+        # 1-fac measurement models: loadings, psi, theta
+        if(start.initial %in% c("lavaan", "mplus") &&
+           model.type %in% c("sem", "cfa") ) {
+            # fabin3 estimator (2sls) of Hagglund (1982) per factor
+            for(f in lv.names) {
+                lambda.idx <- which( lavpartable$lhs == f &
+                                     lavpartable$op == "=~" &
+                                     lavpartable$group == group.values[g] )
+                # standardized?
+                std.lv <- FALSE
+                var.f.idx <- which(lavpartable$lhs == f &
+                                   lavpartable$op == "~~" &
+                                   lavpartable$group == group.values[g] &
+                                   lavpartable$rhs == f)
+                if(length(var.f.idx) > 0L &&
+                   lavpartable$free[var.f.idx] == 0 &&
+                   lavpartable$ustart[var.f.idx] == 1) {
+                    std.lv <- TRUE
+                }
+
+                # no second order
+                if(any(lavpartable$rhs[lambda.idx] %in% lv.names)) next
+
+                # get observed indicators for this latent variable
+                ov.idx <- match(lavpartable$rhs[lambda.idx], ov.names)
+                if(length(ov.idx) > 0L && !any(is.na(ov.idx))) {
+                    if(lavsamplestats@missing.flag && nlevels == 1L) {
+                        COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
+                                                      ov.idx, drop = FALSE]
+                    } else {
+                        if(conditional.x) {
+                            COV <- lavsamplestats@res.cov[[g]][ov.idx,
+                                                      ov.idx, drop = FALSE]
+                        } else {
+                            COV <- lavsamplestats@cov[[g]][ov.idx,
+                                                      ov.idx, drop = FALSE]
+                        }
+                    }
+
+                    # fabin for 1-factor
+                    fabin <- lav_cfa_1fac_fabin(COV, std.lv = std.lv,
+                                                lambda.only = TRUE,
+                                                method = "fabin3")
+
+                    # factor loadings
+                    start[lambda.idx] <- fabin$lambda
+
+                    # factor variance
+                    #if(!std.lv) {
+                    #    start[var.f.idx] <- fabin$psi
+                    #    # if residual var, make smaller
+                    #    y.idx <- which(lavpartable$lhs == f &
+                    #                   lavpartable$group == group.values[g] &
+                    #                   lavpartable$op == "~")
+                    #    if(length(y.idx) > 0L) {
+                    #        # how much explained variance do we expect?
+                    #        # we take 0.50
+                    #        start[var.f.idx] <- 0.5 * start[var.f.idx]
+                    #    }
+                    #    # no negative variances (we get these if we have an
+                    #    # inconsistent triad (eg, covariance signs are +,+,-)
+                    #    if(start[var.f.idx] < 0) {
+                    #        start[var.f.idx] <- 0.05
+                    #    }
+                    #}
+
+                    # NOTE: fabin (sometimes) gives residual variances
+                    # that are larger than the original variances...
+
+                    # residual variances -- order?
+                    #res.idx <- which(lavpartable$lhs %in% ov.names[ov.idx] &
+                    #                 lavpartable$op == "~~" &
+                    #                 lavpartable$group == group.values[g] &
+                    #                 lavpartable$rhs == lavpartable$lhs)
+                    #start[res.idx] <- fabin$theta
+
+                    # negative variances?
+                    #neg.idx <- which(start[res.idx] < 0)
+                    #if(length(neg.idx) > 0L) {
+                    #    start[res.idx][neg.idx] <- 0.05
+                    #}
+                }
+            }
+        } # fabin
+
+        if(model.type == "unrestricted") {
+           # fill in 'covariances' from lavsamplestats
+            cov.idx <- which(lavpartable$group == group.values[g]             &
+                             lavpartable$op    == "~~"          &
+                             lavpartable$lhs != lavpartable$rhs)
+            lhs.idx <- match(lavpartable$lhs[cov.idx], ov.names)
+            rhs.idx <- match(lavpartable$rhs[cov.idx], ov.names)
+            start[cov.idx] <- lavsamplestats@cov[[g]][ cbind(lhs.idx, rhs.idx) ]
+        }
+
+        # variances of ordinal variables - set to 1.0
         if(categorical) {
-            ov.var.ord.idx <- which(lavpartable$block == g            &
+            ov.var.ord.idx <- which(lavpartable$group == group.values[g]            &
                                     lavpartable$op    == "~~"         &
                                     lavpartable$lhs %in% ov.names.ord &
                                     lavpartable$lhs == lavpartable$rhs)
@@ -240,11 +284,11 @@ lav_start <- function(start.method    = "default",
         }
 
         # 3g) intercepts/means
-        ov.int.idx <- which(lavpartable$block == g         &
-                            lavpartable$op == "~1"         & 
+        ov.int.idx <- which(lavpartable$group == group.values[g]         &
+                            lavpartable$op == "~1"         &
                             lavpartable$lhs %in% ov.names)
         sample.int.idx <- match(lavpartable$lhs[ov.int.idx], ov.names)
-        if(lavsamplestats@missing.flag) {
+        if(lavsamplestats@missing.flag && nlevels == 1L) {
             start[ov.int.idx] <- lavsamplestats@missing.h1[[g]]$mu[sample.int.idx]
         } else {
             if(conditional.x) {
@@ -253,68 +297,86 @@ lav_start <- function(start.method    = "default",
                 start[ov.int.idx] <- lavsamplestats@mean[[g]][sample.int.idx]
             }
         }
-        
+
         # 4g) thresholds
-        th.idx <- which(lavpartable$block == g & lavpartable$op == "|")
+        th.idx <- which(lavpartable$group == group.values[g] & lavpartable$op == "|")
         if(length(th.idx) > 0L) {
             th.names.lavpartable <- paste(lavpartable$lhs[th.idx], "|",
                                        lavpartable$rhs[th.idx], sep="")
-            th.names.sample   <- 
+            th.names.sample   <-
                 lavsamplestats@th.names[[g]][ lavsamplestats@th.idx[[g]] > 0L ]
             # th.names.sample should identical to
-            # vnames(lavpartable, "th", block = g)
+            # vnames(lavpartable, "th", group = group.values[g])
             if(conditional.x) {
-                th.values <- 
+                th.values <-
                  lavsamplestats@res.th[[g]][lavsamplestats@th.idx[[g]] > 0L]
             } else {
-                th.values <- 
+                th.values <-
                   lavsamplestats@th[[g]][lavsamplestats@th.idx[[g]] > 0L]
-            }           
+            }
             start[th.idx] <- th.values[match(th.names.lavpartable,
                                              th.names.sample)]
         }
-        
+
         # 5g) exogenous `fixed.x' covariates
-        if(!conditional.x && length(ov.names.x) > 0) {
-            exo.idx <- which(lavpartable$block == g          &
-                             lavpartable$op == "~~"          & 
+        if(length(ov.names.x) > 0) {
+            exo.idx <- which(lavpartable$group == group.values[g]          &
+                             lavpartable$op == "~~"          &
                              lavpartable$lhs %in% ov.names.x &
                              lavpartable$rhs %in% ov.names.x)
-            row.idx <- match(lavpartable$lhs[exo.idx], ov.names)
-            col.idx <- match(lavpartable$rhs[exo.idx], ov.names)
-            if(lavsamplestats@missing.flag) {
-                start[exo.idx] <- 
+            if(!conditional.x) {
+                row.idx <- match(lavpartable$lhs[exo.idx], ov.names)
+                col.idx <- match(lavpartable$rhs[exo.idx], ov.names)
+                if(lavsamplestats@missing.flag && nlevels == 1L) {
+                    start[exo.idx] <-
                     lavsamplestats@missing.h1[[g]]$sigma[cbind(row.idx,col.idx)]
-                # using slightly smaller starting values for free
-                # variance/covariances (fixed.x = FALSE);
-                # this somehow avoids false convergence in saturated models
-                nobs <- lavsamplestats@nobs[[g]]
-                this.idx <- which( seq_len(length(lavpartable$free)) %in% exo.idx & lavpartable$free > 0L )
-                start[this.idx] <- start[this.idx] * (nobs-1)/nobs
+                    # using slightly smaller starting values for free
+                    # variance/covariances (fixed.x = FALSE);
+                    # this somehow avoids false convergence in saturated models
+                    nobs <- lavsamplestats@nobs[[g]]
+                    this.idx <- which( seq_len(length(lavpartable$free)) %in% exo.idx & lavpartable$free > 0L )
+                    start[this.idx] <- start[this.idx] * (nobs-1)/nobs
+                } else {
+                    start[exo.idx] <- lavsamplestats@cov[[g]][cbind(row.idx,col.idx)]
+                }
             } else {
-                start[exo.idx] <- lavsamplestats@cov[[g]][cbind(row.idx,col.idx)]
+                # cov.x
+                row.idx <- match(lavpartable$lhs[exo.idx], ov.names.x)
+                col.idx <- match(lavpartable$rhs[exo.idx], ov.names.x)
+                start[exo.idx] <- lavsamplestats@cov.x[[g]][cbind(row.idx,
+                                                                  col.idx)]
+                # mean.x
+                exo.int.idx <- which(lavpartable$group == group.values[g] &
+                                     lavpartable$op == "~1"               &
+                                     lavpartable$lhs %in% ov.names.x)
+                int.idx <- match(lavpartable$lhs[exo.int.idx], ov.names.x)
+                start[exo.int.idx] <- lavsamplestats@mean.x[[g]][int.idx]
             }
         }
 
         # 6b. exogenous lv variances if single indicator -- new in 0.5-21
-        lv.x <- vnames(lavpartable, "lv.x", block = g)
+        lv.x <- vnames(lavpartable, "lv.x", group = group.values[g])
+        # FIXME: also for multilevel?
+        lv.x <- unique(unlist(lv.x))
         if(length(lv.x) > 0L) {
             for(ll in lv.x) {
                 ind.idx <- which(lavpartable$op == "=~" &
                                  lavpartable$lhs == ll,
-                                 lavpartable$block == g)
+                                 lavpartable$group == group.values[g])
                 if(length(ind.idx) == 1L) {
                     single.ind <- lavpartable$rhs[ind.idx]
                     single.fvar.idx <- which(lavpartable$op == "~~" &
                                              lavpartable$lhs == ll &
                                              lavpartable$rhs == ll &
-                                             lavpartable$block == g)
+                                             lavpartable$group == group.values[g])
                     single.var.idx <- which(lavpartable$op == "~~" &
                                             lavpartable$lhs == single.ind &
                                             lavpartable$rhs == single.ind &
-                                            lavpartable$block == g)
+                                            lavpartable$group == group.values[g])
                     # user-defined residual variance
-                    single.var <- lavpartable$ustart[single.var.idx]
+                    # fixme: we take the first, in case we have multiple matches
+                    # (eg nlevels)
+                    single.var <- lavpartable$ustart[single.var.idx[1]]
                     if(is.na(single.var)) {
                          single.var <- 1
                     }
@@ -335,9 +397,124 @@ lav_start <- function(start.method    = "default",
             }
         }
 
-
         # 7g) regressions "~"
-    }
+
+      #  # 8 latent variances (new in 0.6-2)
+      #  lv.names.y <- vnames(lavpartable, "lv.y", group = group.values[g])
+      #  lv.names.x <- vnames(lavpartable, "lv.x", group = group.values[g])
+      #  # multilevel? take first level only
+      #  if(is.list(lv.names.y)) {
+      #      lv.names.y <- unlist(lv.names.y) # for now
+      #  }
+      #  if(is.list(lv.names.x)) {
+      #      lv.names.x <- unlist(lv.names.x) # for now
+      #  }
+      #  lv.names.xy <- unique(c(lv.names.x, lv.names.y))
+
+
+      #  if(length(lv.names.xy) > 0L) {
+      #      free.var.idx <- which(lavpartable$op == "~~" &
+      #                            lavpartable$lhs %in% lv.names.xy &
+      #                            lavpartable$rhs == lavpartable$lhs &
+      #                            lavpartable$group == group.values[g])
+      #      if(length(free.var.idx) > 0L) {
+      #          this.lv.names <- lavpartable$lhs[free.var.idx]
+      #          for(v in seq_len(length(free.var.idx))) {
+      #              # single marker item?
+      #              ind.idx <- which(lavpartable$op == "=~" &
+      #                               lavpartable$lhs %in% this.lv.names[v] &
+      #                               #lavpartable$rhs %in% ov.names.num &
+      #                               lavpartable$free == 0L &
+      #                               lavpartable$group == group.values[g])
+      #              if(length(ind.idx) == 0) {
+      #                  next
+      #              } else if(length(ind.idx) > 1L) {
+      #                  # FIXME! perhaps a random effect? do something clever
+      #                  next
+      #              } else if(length(ind.idx) == 1L) {
+      #                  marker.ind <- lavpartable$rhs[ind.idx]
+      #                  ov.idx <- match(marker.ind, ov.names)
+      #                  if(conditional.x) {
+      #                      ov.var <- diag(lavsamplestats@res.cov[[g]])[ov.idx]
+      #                  } else {
+      #                      ov.var <- diag(lavsamplestats@cov[[g]])[ov.idx]
+      #                  }
+      #
+      #                  # exogenous? assume rel = 0.50
+      #                  lambda <- lavpartable$ustart[ind.idx]
+      #                  tmp <- (0.50 * ov.var)/lambda^2
+      #                  if(this.lv.names[v] %in% lv.names.y) {
+      #                      # endogenous, assume R2 = 0.2
+      #                      tmp <- 0.8 * tmp
+      #                  }
+      #                  # within variance?
+      #                  if(nlevels > 1L &&
+      #                     lavpartable$level[ free.var.idx[v] ] == 1L) {
+      #                      tmp <- tmp * 0.75
+      #                  }
+      #                  # between variance?
+      #                  if(nlevels > 1L &&
+      #                     lavpartable$level[ free.var.idx[v] ] > 1L) {
+      #                      tmp <- tmp * 0.25
+      #                  }
+      #                  # just in case
+      #                  if(is.na(tmp) || tmp < 0.05) {
+      #                      tmp <- 0.05
+      #                  }
+      #                  start[ free.var.idx[v] ] <- tmp
+      #              }
+      #          } # v
+      #      } # free.var.idx
+      #  } # lv var
+
+    } # groups
+
+
+    # nlevels > 1L
+    if(nlevels > 1L) {
+        for(g in 1:ngroups) {
+            group.values <- lav_partable_group_values(lavpartable)
+            ov.names.x  <- vnames(lavpartable, "ov.x", group = group.values[g])
+            ov.names.x <- unique(unlist(ov.names.x))
+            level.values <- lav_partable_level_values(lavpartable)
+
+            if(!conditional.x && length(ov.names.x) > 0) {
+                for(l in 1:nlevels) {
+                    # var/cov
+                    exo.idx <- which(lavpartable$group == group.values[g] &
+                                     lavpartable$level == level.values[l] &
+                                     lavpartable$op == "~~"               &
+                                     lavpartable$lhs %in% ov.names.x      &
+                                     lavpartable$rhs %in% ov.names.x)
+                    row.idx <- match(lavpartable$lhs[exo.idx], ov.names)
+                    col.idx <- match(lavpartable$rhs[exo.idx], ov.names)
+
+                    if(l == 1L) {
+                        COV <- lavsamplestats@YLp[[g]][[2]]$S.PW.start
+                    } else {
+                        COV <- lavsamplestats@YLp[[g]][[l]]$Sigma.B
+                    }
+                    start[exo.idx] <- COV[ cbind(row.idx, col.idx) ]
+
+                    # intercepts
+                    ov.int.idx <- which(lavpartable$group == group.values[g] &
+                                        lavpartable$level == level.values[l] &
+                                        lavpartable$op == "~1"               &
+                                        lavpartable$lhs %in% ov.names.x)
+                    idx <- match(lavpartable$lhs[ov.int.idx], ov.names)
+
+                    if(l == 1L) {
+                        INT <- lavsamplestats@YLp[[g]][[2]]$Mu.W
+                    } else {
+                        INT <- lavsamplestats@YLp[[g]][[l]]$Mu.B.start
+                    }
+
+                    start[ov.int.idx] <- INT[idx]
+                } # levels
+            } # fixed.x
+        } # groups
+    } # nlevels > 1L
+
 
     # group weights
     group.idx <- which(lavpartable$lhs == "group" &
@@ -355,48 +532,48 @@ lav_start <- function(start.method    = "default",
     # growth models:
     # - compute starting values for mean latent variables
     # - compute starting values for variance latent variables
-    if(start.initial %in% c("lavaan", "mplus") && 
+    if(start.initial %in% c("lavaan", "mplus") &&
        model.type == "growth") {
         ### DEBUG ONLY
         #lv.var.idx <- which(lavpartable$op == "~~"                &
         #                lavpartable$lhs %in% lv.names &
         #                lavpartable$lhs == lavpartable$rhs)
-        
+
         ### DEBUG ONLY
         #lv.int.idx <- which(lavpartable$op == "~1"         &
         #                    lavpartable$lhs %in% lv.names)
     }
 
-    # override if a user list with starting values is provided 
+    # override if a user list with starting values is provided
     # we only look at the 'est' column for now
     if(!is.null(start.user)) {
 
-        if(is.null(lavpartable$block)) {
-            lavpartable$block <- rep(1L, length(lavpartable$lhs))
+        if(is.null(lavpartable$group)) {
+            lavpartable$group <- rep(1L, length(lavpartable$lhs))
         }
-        if(is.null(start.user$block)) {
-            start.user$block <- rep(1L, length(start.user$lhs))
+        if(is.null(start.user$group)) {
+            start.user$group <- rep(1L, length(start.user$lhs))
         }
 
         # FIXME: avoid for loop!!!
         for(i in 1:length(lavpartable$lhs)) {
             # find corresponding parameters
             lhs <- lavpartable$lhs[i]
-             op <- lavpartable$op[i] 
+             op <- lavpartable$op[i]
             rhs <- lavpartable$rhs[i]
-            grp <- lavpartable$block[i]
+            grp <- lavpartable$group[i]
 
             start.user.idx <- which(start.user$lhs == lhs &
                                     start.user$op  ==  op &
                                     start.user$rhs == rhs &
-                                    start.user$block == grp)
-            if(length(start.user.idx) == 1L && 
+                                    start.user$group == grp)
+            if(length(start.user.idx) == 1L &&
                is.finite(start.user$est[start.user.idx])) {
                 start[i] <- start.user$est[start.user.idx]
             }
         }
     }
-  
+
     # override if the model syntax contains explicit starting values
     user.idx <- which(!is.na(lavpartable$ustart))
     start[user.idx] <- lavpartable$ustart[user.idx]
@@ -419,15 +596,18 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
 
     for(g in 1:nblocks) {
 
+        # block values
+        block.values <- lav_partable_block_values(lavpartable)
+
         # collect all non-zero covariances
         cov.idx <- which(lavpartable$op == "~~" &
-                         lavpartable$block == g &
+                         lavpartable$block == block.values[g] &
                          lavpartable$lhs != lavpartable$rhs &
                          !lavpartable$exo &
                          start != 0)
 
         # for each covariance, use corresponding variances to standardize;
-        # the end result should not exceed abs(1)     
+        # the end result should not exceed abs(1)
         for(cc in seq_along(cov.idx)) {
             this.cov.idx <- cov.idx[cc]
 
@@ -436,12 +616,12 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
             var.rhs <- lavpartable$rhs[this.cov.idx]
 
             var.lhs.idx <- which(lavpartable$op == "~~" &
-                                 lavpartable$block == g &
+                                 lavpartable$block == block.values[g] &
                                  lavpartable$lhs == var.lhs &
                                  lavpartable$lhs == lavpartable$rhs)
 
             var.rhs.idx <- which(lavpartable$op == "~~" &
-                                 lavpartable$block == g &
+                                 lavpartable$block == block.values[g] &
                                  lavpartable$lhs == var.rhs &
                                  lavpartable$lhs == lavpartable$rhs)
 
@@ -452,11 +632,11 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
             if(nblocks > 1L) {
                 block.txt <- paste(" [in block ", g, "]", sep = "")
             }
- 
+
             # check for zero variances
             if(var.lhs.value == 0 || var.rhs.value == 0) {
                 # this can only happen if it is user-specified
-                # cov.idx free? set it to zero    
+                # cov.idx free? set it to zero
                 if(start[this.cov.idx] == 0) {
                     # nothing to do
                 } else if(lavpartable$free[this.cov.idx] > 0L) {
@@ -482,20 +662,20 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
 
             # check
             COR <- start[this.cov.idx] / sqrt(var.lhs.value * var.rhs.value)
- 
+
             if(!is.finite(COR)) {
                 # force simple values
                 warning(
   "lavaan WARNING: starting values imply NaN for a correlation value;\n",
-"                  variables involved are: ", var.lhs, " ", var.rhs, block.txt)    
+"                  variables involved are: ", var.lhs, " ", var.rhs, block.txt)
                 start[var.lhs.idx] <- 1
                 start[var.rhs.idx] <- 1
                 start[this.cov.idx] <- 0
             } else if(abs(COR) > 1) {
                 warning(
-  "lavaan WARNING: starting values imply a correlation larger than 1;\n", 
+  "lavaan WARNING: starting values imply a correlation larger than 1;\n",
 "                  variables involved are: ", var.lhs, " ", var.rhs, block.txt)
-                
+
                 # three ways to fix it: rescale cov12, var1 or var2
 
                 # we prefer a free parameter, and not user-specified
